#include "tridiagonalmatrix.hpp"

TridiagonalMatrix::TridiagonalMatrix(int s)
{
    if (s < 1) // ha hibás méretet adunk meg
        throw WRONGSIZE;
    size = s;
    values = new int[3 * size - 2]; // megfelelõ méretû mátrix létrehozása
    for (int i = 0; i < 3 * size - 2; i++)
        values[i] = 0; // kezdetben minden érték 0
}

TridiagonalMatrix::TridiagonalMatrix(const TridiagonalMatrix& other)
{
    size = other.size; // átvesszük a másik mátrix attribútumait
    values = new int[3 * size - 2];
    for (int i = 0; i < 3 * size - 2; i++)
        values[i] = other.values[i];
}

TridiagonalMatrix::~TridiagonalMatrix()
{
    delete[] values; // tömb törlése
}

const TridiagonalMatrix& TridiagonalMatrix::operator=(const TridiagonalMatrix& other)
{
    if (this == &other) // ha ugyanaz, visszatérés
        return *this;

    delete[] values; // töröljük a korábbi értékeket

    size = other.size; // felvesszük az újakat
    values = new int[3 * size - 2];
    for (int i = 0; i < 3 * size - 2; i++)
        values[i] = other.values[i];

    return *this;
}

int TridiagonalMatrix::operator()(int i, int j) const
{
    return GetValue(i, j); // elég a másik lekérdező műveletet meghívni
}

int& TridiagonalMatrix::operator()(int i, int j)
{
    if (i < 0 || i >= size || j < 0 || j >= size)
        throw INDEXOUTOFRANGE;
    else
        if (j >= i - 1 && j <= i + 1) // ha a megfelelõ értékek közé akarunk beírni
            return values[2 * i + j]; // ha referencia szerint adjuk vissza az értéket, akkor beállíthatjuk
        else
            throw NULLREGION; // ha nem a megfelelõ értékek közé akarunk beírni
}

void TridiagonalMatrix::Resize(int s)
{
    if (s < 1)
        throw WRONGSIZE;
    int temp_size = s; // az értékeket elõször átmeneti helyre helyezzük az átmérezetéshez
    int* temp_values = new int[3 * size - 2];

    for (int i = 0; i < size && i < temp_size; i++)
        temp_values[i] = values[i]; // a régi értékeket átvesszük
    for (int i = size; i < temp_size; i++)
        temp_values[i] = 0; // az új helyekre 0-t írunk

    delete[] values; // régi értékek törlése
    values = temp_values; // új értékek rögzítése
    size = temp_size;
}

int TridiagonalMatrix::GetValue(int i, int j) const
{
    if (i < 0 || i >= size || j < 0 || j >= size) // túlindexelés
        throw INDEXOUTOFRANGE;
    else
        if (j >= i - 1 && j <= i + 1) // ha az értékes részben kérdezünk le
            return values[2 * i + j]; // kiszámítjuk a megfelelõ elemet
        else
            return 0; // különben 0
}

void TridiagonalMatrix::SetValue(int i, int j, int value)
{
    if (i < 0 || i >= size || j < 0 || j >= size)
        throw INDEXOUTOFRANGE;
    else
        if (j >= i - 1 && j <= i + 1) // ha a megfelelõ értékek közé akarunk beírni
            values[2 * i + j] = value;
        else
            throw NULLREGION; // ha nem a megfelelõ értékek közé akarunk beírni
}

TridiagonalMatrix TridiagonalMatrix::operator-() const
{
    TridiagonalMatrix result(size);

    for (int i = 0; i < 3 * size - 2; i++)
        result.values[i] = -values[i];

    return result; // visszaadjuk a negált mátrixot
}

TridiagonalMatrix operator+ (const TridiagonalMatrix& a, const TridiagonalMatrix& b)
{
    if (a.size != b.size)
        throw TridiagonalMatrix::DIFFERENTSIZE;
    else
    {
        TridiagonalMatrix c(a.size); // ebbe helyezzük az eredményt
        for (int i = 0; i < 3 * c.size - 2; i++)
            c.values[i] = a.values[i] + b.values[i]; // megfelelõ értékeket összeadjuk
        return c;
    }
}
TridiagonalMatrix operator- (const TridiagonalMatrix& a, const TridiagonalMatrix& b)
{
    if (a.size != b.size)
        throw TridiagonalMatrix::DIFFERENTSIZE;
    else
    {
        TridiagonalMatrix c(a.size);
        for (int i = 0; i < 3 * c.size - 2; i++)
            c.values[i] = a.values[i] - b.values[i];
        return c;
    }
}

bool operator==(const TridiagonalMatrix& a, const TridiagonalMatrix& b)
{
    if (&a == &b) // ha a két mátrix ugyanaz (ugyanazon a memóriahelyen található)
        return true;

    bool equal = (a.size == b.size);
    for (int i = 0; (i < 3 * a.size - 2) && equal; i++)
        equal = (a.values[i] == b.values[i]);

    return equal;
}

std::istream& operator>>(std::istream& s, TridiagonalMatrix& a)
{
    for (int i = 0; i < 3 * a.size - 2; i++)
    {
        s >> a.values[i]; // értékek bekérése sorrendben

        if (s.fail()) // ha nem sikerült a beolvasás, kinullázzuk az értéket
        {
            s.clear();
            a.values[i] = 0;
        }
    }

    return s;
}
std::ostream& operator<<(std::ostream& s, const TridiagonalMatrix& a)
{
    // a kiírás mátrix alakban történik
    for (int i = 0; i < a.size; i++)
    {
        int j = 0;
        while (j < i - 1)
        {
            s << "0\t"; // megfelelõ számú 0 a sor elején
            j++;
        }
        while (j < i + 2 && j < a.size)
        {
            s << a.values[2 * i + j] << "\t"; // tridiagonális értékek kiírása
            j++;
        }
        while (j < a.size)
        {
            s << "0\t"; // megfelelõ számú 0 a sor végén
            j++;
        }
        s << std::endl;
    }

    return s;
}
